#include "stdafx.h"
#include "pchk.h"
#include "pchkDlg.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

#define BLANK_TEXT	(_T(""))
#define PASSWORD_CHAR	(_T('*'))
#define PROGRESS_MIN	(0)
#define PROGRESS_MAX	(50)
#define PROGRESS_STEP	(1)
#define PROGRESS_INIT	(0)

CPchkDlg::CPchkDlg(CWnd* pParent /*=NULL*/)
	: CDialog(CPchkDlg::IDD, pParent)
{
	//{{AFX_DATA_INIT(CPchkDlg)
	m_strStrength = _T("");
	m_bCheckShow = FALSE;
	//}}AFX_DATA_INIT
	m_hIcon = AfxGetApp()->LoadIcon(IDR_MAINFRAME);
}

void CPchkDlg::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CPchkDlg)
	DDX_Control(pDX, IDC_EDIT_PASSWORD, m_xcEditPassword);
	DDX_Control(pDX, IDC_PROGRESS_STRENGTH, m_xcProgStrength);
	DDX_Text(pDX, IDC_STATIC_STRENGTH, m_strStrength);
	DDV_MaxChars(pDX, m_strStrength, 4);
	DDX_Check(pDX, IDC_CHECK_SHOW, m_bCheckShow);
	//}}AFX_DATA_MAP
}

BEGIN_MESSAGE_MAP(CPchkDlg, CDialog)
	//{{AFX_MSG_MAP(CPchkDlg)
	ON_WM_PAINT()
	ON_WM_QUERYDRAGICON()
	ON_BN_CLICKED(IDC_CHECK_SHOW, OnCheckShow)
	ON_EN_CHANGE(IDC_EDIT_PASSWORD, OnChangeEditPassword)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

BOOL CPchkDlg::OnInitDialog()
{
	CDialog::OnInitDialog();

	SetIcon(m_hIcon, TRUE);
	SetIcon(m_hIcon, FALSE);
	
	m_bCheckShow = FALSE;
	m_strStrength = BLANK_TEXT;
	UpdateData(FALSE);

	m_xcEditPassword.SetWindowText(BLANK_TEXT);
	m_xcEditPassword.SetPasswordChar(PASSWORD_CHAR);

	m_xcProgStrength.SetRange(PROGRESS_MIN, PROGRESS_MAX);
	m_xcProgStrength.SetStep(PROGRESS_STEP);
	m_xcProgStrength.SetPos(PROGRESS_INIT);

	return TRUE;
}

void CPchkDlg::OnPaint() 
{
	if (IsIconic())
	{
		CPaintDC dc(this);

		SendMessage(WM_ICONERASEBKGND, (WPARAM) dc.GetSafeHdc(), 0);

		int cxIcon = GetSystemMetrics(SM_CXICON);
		int cyIcon = GetSystemMetrics(SM_CYICON);
		CRect rect;
		GetClientRect(&rect);
		int x = (rect.Width() - cxIcon + 1) / 2;
		int y = (rect.Height() - cyIcon + 1) / 2;

		dc.DrawIcon(x, y, m_hIcon);
	}
	else
	{
		CDialog::OnPaint();
	}
}

HCURSOR CPchkDlg::OnQueryDragIcon()
{
	return (HCURSOR) m_hIcon;
}

int CPchkDlg::CheckStrength(const CString strPassword)
{
	int nScore	= 0;
	int nLength	= strPassword.GetLength();
	int nDigit	= 0;
	int nUpper	= 0;
	int nLower	= 0;
	int nSpChr	= 0;
	TCHAR ch;
	
	// length check
	if (nLength >= 0 && nLength <= 4) {
		nScore += nLength;
	} else if (nLength >= 5 && nLength <= 7) {
		nScore += 6;
	} else if (nLength >= 8 && nLength <= 15) {
		nScore += 12;
	} else if (nLength >= 16) {
		nScore += 18;
	}

	// character check
	if (nLength > 0) {
		for (int i = 0; i < nLength; i++) {
			ch = strPassword.GetAt(i);

			if (::iswlower((int)ch)) {
				nLower++;
			} else if (::iswupper((int)ch)) {
				nUpper++;
			} else if (::iswdigit((int)ch)) {
				nDigit++;
			} else if (!::isalnum((int)ch)) {
				nSpChr++;
			}
		}

		nScore += (nLower > 0) ? 1 : 0;
		nScore += (nUpper > 0) ? 5 : 0;
		nScore += (nDigit > 0) ? 5 : 0;
		nScore += (nSpChr > 0) ? 5 : 0;

		nScore += (nDigit >= 3) ? 5 : 0;
		nScore += (nSpChr >= 2) ? 5 : 0;

		nScore += (nLower > 0 && nUpper > 0) ? 2 : 0;
		nScore += (nLower > 0 && nUpper > 0 && nDigit > 0) ? 2 : 0;
		nScore += (nLower > 0 && nUpper > 0 && nDigit > 0 && nSpChr > 0) ? 2 : 0;
	}

	return nScore;
}

void CPchkDlg::OnCheckShow() 
{
	UpdateData(TRUE);

	if (m_bCheckShow) {
		m_xcEditPassword.SetPasswordChar(0);
	} else {
		m_xcEditPassword.SetPasswordChar(PASSWORD_CHAR);
	}
	Invalidate();

	return;
}

void CPchkDlg::OnChangeEditPassword() 
{
	CString strPassword;
	int nScore;
	
	m_xcEditPassword.GetWindowText(strPassword);
	nScore = CheckStrength(strPassword);

	m_xcProgStrength.SetPos(nScore);

	if (nScore > 0) {
		m_strStrength.Format("%d", nScore);
	} else {
		m_strStrength = BLANK_TEXT;
	}
	UpdateData(FALSE);

	return;	
}
